package com.sun.xml.internal.ws.message;

import javax.xml.namespace.QName;
import java.util.Set;
import com.sun.istack.internal.NotNull;
import com.sun.xml.internal.ws.api.SOAPVersion;
import javax.xml.stream.XMLStreamReader;
import com.sun.xml.internal.ws.api.streaming.XMLStreamReaderFactory;
import com.sun.xml.internal.ws.api.addressing.WSEndpointReference;
import com.sun.xml.internal.ws.api.addressing.AddressingVersion;
import com.sun.xml.internal.ws.spi.db.XMLBridge;
import javax.xml.stream.XMLStreamException;
import javax.xml.bind.Unmarshaller;
import javax.xml.bind.JAXBException;
import com.sun.xml.internal.bind.api.BridgeContext;
import com.sun.xml.internal.bind.api.Bridge;
import org.xml.sax.helpers.AttributesImpl;
import com.sun.xml.internal.ws.api.message.Header;

public abstract class AbstractHeaderImpl implements Header
{
    protected static final AttributesImpl EMPTY_ATTS;
    
    protected AbstractHeaderImpl() {
    }
    
    @Deprecated
    public final <T> T readAsJAXB(Bridge<T> bridge, BridgeContext context) throws JAXBException {
        return this.readAsJAXB(bridge);
    }
    
    public <T> T readAsJAXB(Unmarshaller unmarshaller) throws JAXBException {
        try {
            return unmarshaller.unmarshal(this.readHeader());
        }
        catch (Exception e) {
            throw new JAXBException(e);
        }
    }
    
    @Deprecated
    public <T> T readAsJAXB(Bridge<T> bridge) throws JAXBException {
        try {
            return bridge.unmarshal(this.readHeader());
        }
        catch (XMLStreamException e) {
            throw new JAXBException(e);
        }
    }
    
    public <T> T readAsJAXB(XMLBridge<T> bridge) throws JAXBException {
        try {
            return bridge.unmarshal(this.readHeader(), null);
        }
        catch (XMLStreamException e) {
            throw new JAXBException(e);
        }
    }
    
    public WSEndpointReference readAsEPR(AddressingVersion expected) throws XMLStreamException {
        XMLStreamReader xsr = this.readHeader();
        WSEndpointReference epr = new WSEndpointReference(xsr, expected);
        XMLStreamReaderFactory.recycle(xsr);
        return epr;
    }
    
    public boolean isIgnorable(@NotNull SOAPVersion soapVersion, @NotNull Set<String> roles) {
        String v = this.getAttribute(soapVersion.nsUri, "mustUnderstand");
        return v == null || !this.parseBool(v) || roles == null || !roles.contains(this.getRole(soapVersion));
    }
    
    @NotNull
    public String getRole(@NotNull SOAPVersion soapVersion) {
        String v = this.getAttribute(soapVersion.nsUri, soapVersion.roleAttributeName);
        if (v == null) {
            v = soapVersion.implicitRole;
            return v;
        }
        return v;
    }
    
    public boolean isRelay() {
        String v = this.getAttribute(SOAPVersion.SOAP_12.nsUri, "relay");
        return v != null && this.parseBool(v);
    }
    
    public String getAttribute(QName name) {
        return this.getAttribute(name.getNamespaceURI(), name.getLocalPart());
    }
    
    protected final boolean parseBool(String value) {
        if (value.length() == 0) {
            return false;
        }
        char ch = value.charAt(0);
        return ch == 't' || ch == '1';
    }
    
    public String getStringContent() {
        try {
            XMLStreamReader xsr = this.readHeader();
            xsr.nextTag();
            return xsr.getElementText();
        }
        catch (XMLStreamException e) {
            return null;
        }
    }
    
    static {
        EMPTY_ATTS = new AttributesImpl();
    }
}
